@@ -5,18 +5,14 @@ use warnings;
 use strict;
 
 package goat;
+BEGIN {
+  $goat::VERSION = '0.04';
+}
 use Oogly::Aagly ();
 use File::ShareDir ':ALL';
 use File::Copy;
 use Cwd;
 
-=method copy_templates
-
-The copy_templates method utilizied by goat.pl (get Oogly-Aagly Templates), 
-will copy the default form element templates stored in the main Perl library, 
-to the current working directory.
-
-=cut
 
 sub copy_templates {
     my $to   = Cwd::getcwd();
@@ -42,4 +38,36 @@ sub copy_templates {
 # copy standard template to the cwd
 copy_templates;
 
-1;
\ No newline at end of file
+1;
+__END__
+=pod
+
+=head1 NAME
+
+goat - Get Oogly-Aagly Templates
+
+=head1 VERSION
+
+version 0.04
+
+=head1 METHODS
+
+=head2 copy_templates
+
+The copy_templates method utilizied by goat.pl (get Oogly-Aagly Templates), 
+will copy the default form element templates stored in the main Perl library, 
+to the current working directory.
+
+=head1 AUTHOR
+
+Al Newkirk <awncorp@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Al Newkirk.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
+
